% planaraf.m
%
% This program models point source planar arrays. 
% planar array is in x-y plane with physical center at (0,0)
% 
% Antenna Arrays a Computational Approach
% Author: Randy L. Haupt
% 2010
% This program is only intended for demonstrations of book examples.
%

clear
% constants
f=3*10^8; c=3*10^8; lam=c/f; k=2*pi/lam;

%%%%%%%%%%%%  element positions  %%%%%%%%%%%%
sp=1; %=1 rectangular grid; =2 triangular grid
%%%%%%%%%%%%  array shape  %%%%%%%%%%%%
sh=1; % array shape 1= rectangle; 2=octagon; 3=circle
%%%%%%%%%%%%  element weights  %%%%%%%%%%%%
wt=1; % amplitude taper 1=uniform; 2=Taylor; 3=Hansen 1 par; 4=Tseng; 5=cir Bayliss 6=separable
      % 7=radial taper  8=radial taper squared
%%%%%%%%%%%%  element thinning  %%%%%%%%%%%%
thin=0; % 0=no thinning; 1=thinning
%%%%%%%%%%%%  element pattern  %%%%%%%%%%%%
eltype=1; % 1=isotropic pt sources 2=hertzian dipoles, 3=patch ; 4=helix
          % 5=uniform rect slot, 6=uniform circular slot, 7=TE10 wg
          % 8=E-plane horn, 9=H-plane horn, 10=pyramidal horn
%%%%%%%%%%%%  plotting  %%%%%%%%%%%%
azel=2; % pattern plot =1 z-theta; =2 theta,phi; =3 u-v
elplt=[0 0 0]; % [element locations in plane; stem plot of weights; surf plot of wgts] 1=yes; 0=no
afplt=[1 1 1 1 1]; %[]
ff=1; % figure number

%%%%%%%%%%%%  element positions  %%%%%%%%%%%%
dx=1*lam; dy=sqrt(3)/2*lam;
Nx=3; Ny=3;
x0=[0:dx:(Nx-1)*dx];
y0=[0:dy:(Ny-1)*dy];
% rectangular grid
if sp==1
    [xx,yy]=meshgrid(x0,y0);
    x=reshape(xx,1,Nx*Ny);
    y=reshape(yy,1,Nx*Ny);
    w=zeros(1,Ny*Nx);
    Nt=Nx*Ny;
% triangular grid
elseif sp==2
    N1x=Nx; N2x=Nx; N1y=Ny; N2y=Ny;
    [x1,y1]=meshgrid(x0,y0);
    [x2,y2]=meshgrid(x0(1:N2x)+dx/2,y0(1:N2y)+dy/2);
    x=[reshape(x1,1,N1x*N1y) reshape(x2,1,N2x*N2y)];
    y=[reshape(y1,1,N1x*N1y) reshape(y2,1,N2x*N2y)];
    w=zeros(1,N1x*N1y+N2x*N2y);
    Nt=N1x*N1y+N2x*N2y;
end
x=x-max(x)/2; y=y-max(y)/2; % centers array at (0,0)

%%%%%%%%%%%%  array shape  %%%%%%%%%%%%
if sh==1
% rectangle
    p=find(abs(y)<=max(y) & abs(x)<=max(x)); %square
    area=Nt^2*dx*dy;
elseif sh==2
% hexagon
    sq=Nx*dx; sd=sq/(1+2*tan(pi/4));
    p=find(y>=x-sq/2-sd/2 & y>=-x-sq/2-sd/2 & y<=x+sq/2+sd/2 & y<=-x+sq/2+sd/2 ); % octagon
    area=1.5*sqrt(3)*sd^2;
elseif sh==3
%ellipse
    a=.501*abs(max(x)-min(x));
    b=.501*abs(max(y)-min(y));
    p=find(sqrt((x/a).^2+(y/b).^2)<=1);
    area=pi*a*b;
end
xc=x(p); yc=y(p);
Nc=length(xc);
rx=max(xc); ry= max(yc);

%%%%%%%%%%%%  element weights  %%%%%%%%%%%%
% Uniform
if wt==1
    w(p)=ones(1,Nc);
    % circular Taylor
elseif wt==2
    for ic=1:Nc
        aa=atan2(yc(ic),xc(ic));
        r=sqrt((rx*cos(aa))^2+(ry*sin(aa))^2);
        w(p(ic))=taycirpt(sqrt(xc(ic)^2+yc(ic)^2)/r,30,5);
    end
% Hansen 1 parameter
elseif wt==3
    for ic=1:Nc
        aa=atan2(yc(ic),xc(ic));
        r=sqrt((rx*cos(aa))^2+(ry*sin(aa))^2);
        w(p(ic))=hansen(sqrt(xc(ic)^2+yc(ic)^2)/r,30);
    end
% Tseng
elseif wt==4 & sh==1 & sp==1
    ww=tseng(Nx,30);
    w=reshape(ww,1,Nt);
% circular Bayliss
elseif wt==5
    aa=atan2(yc(ic),xc(ic));
    r=sqrt((rx*cos(aa))^2+(ry*sin(aa))^2);
    w(p)=baycirpt(sqrt(xc.^2+yc.^2)/r,30,5).*xc./sqrt(xc.^2+yc.^2+.000001);
elseif wt==6
%separable
    wx=acheby(Nx/2,20,dx);
    wy=acheby(Ny/2,30,dy);
    ww=wy'*wx;
    w=reshape(ww,1,Nt);
% radial taper
elseif wt==7
    w=1-(x.^2+y.^2)/r^2;
% radial taper squared
elseif wt==8
    w=(1-(x.^2+y.^2)/r^2).^2;
elseif wt==9
    w=carp;
end
w=w/max(max(w));
wc=w(p);

%%%%%%%%%%%%  element thinning  %%%%%%%%%%%%
if thin==1
    for ic=1:Nc
        if w(p(ic))>rand
            w(p(ic))=1;
            wc(ic)=1;
        else
            w(p(ic))=0;
            wc(ic)=0;
        end
    end
end

%%%%%%%%%%%%  taper efficiency  %%%%%%%%%%%%
te=abs(sum(wc))^2/sum(abs(wc).^2)/Nc;

%%%%%%%%%%%%  far field angles  %%%%%%%%%%%%
% az-el angles
if Nx*Ny==1
nang=81;
else
nang=10*max([Nx Ny])+1;
end
na2=nang^2;
if azel==1
    samps=-1.001:2/(nang-1):1.001;
    [xx,yy]=meshgrid(samps,samps);
    ther=sqrt(xx.^2+yy.^2)*pi/2;
    phir=atan2(yy,xx);
else
    the=0:90/(nang-1):90;
    phi=0:360/(nang-1):360;
    [ther,phir]=meshgrid(the,phi);
    ther=ther*pi/180; phir=phir*pi/180;
end
trow=reshape(ther,1,na2);
u = reshape((sin(ther).*cos(phir)),1,na2);
v = reshape((sin(ther).*sin(phir)),1,na2);
% steering angles
thes=0; thesr=thes*pi/180;
phis=0; phisr=phis*pi/180;
us=sin(thesr)*cos(phisr);
vs=sin(thesr)*sin(phisr);

%%%%%%%%%%%%  calculate array factor  %%%%%%%%%%%%
af=wc*exp(j*k*(xc'*(u-us)+yc'*(v-vs)));
afa=abs(af);
qq=find(trow>pi/2);
afa(qq)=0;
maxaf=max(afa);
afe=abs(afa.*elementpat(trow,reshape(phir,1,na2),eltype));
afr=reshape(afe,nang,nang);

%%%%%%%%%%%%  array factor directivity  %%%%%%%%%%%%
num=4*pi*abs(elementpat(thesr,phisr,eltype)*maxaf)^2;
maxp=2*pi; maxt=pi/2;
np=nang; nt=nang;
dp=maxp/np; dt=maxt/nt;
pp=dp/2:dp:maxp;
tt=dt/2:dt:maxt;
[t p]=meshgrid(tt,pp);
t=reshape(t,1,np*nt); p=reshape(p,1,np*nt);
ss0=wc*exp(j*k*(xc'*(sin(t).*cos(p)-sin(thesr)*cos(phisr))+yc'*(sin(t).*sin(p)-sin(thesr)*sin(phisr))));
ss1=ss0.*elementpat(t,p,eltype);
den=dp*dt*sum(abs(ss1).^2.*sin(t));
D=num/den;
Ddb=10*log10(D);

%%%%%%%%%%%% AF for plotting  %%%%%%%%%%%%
afdb=20*log10(afe/maxaf);
bot=-40;
afdb(find(afdb<bot)) =bot;
afdbp=reshape(afdb,nang,nang)-bot;

%%%%%%%%%%%%  plot element info  %%%%%%%%%%%%
if elplt(1)==1
    if thin==0
        figure(ff);plot(xc,yc,'o',xc,yc,'*');axis square;axis equal;xlabel('x');ylabel('y')
    else
        figure(ff);plot(xc(wc>0),yc(wc>0),'o',xc(wc>0),yc(wc>0),'*');axis square;axis equal;xlabel('x');ylabel('y')
    end
    if sh==3
        hold on;plot(a*cos([0:.01:2*pi]),b*sin([0:.01:2*pi]));hold off
    end
    ff=ff+1;
end
if elplt(2)==1
    if thin==0
        if sp==1
            figure(ff);stem3(xc,yc,wc)
        else
            figure(ff);stem3(xc,yc,wc)
        end
    else
        if sp==1
            figure(ff);stem3(xc(wc>0),yc(wc>0),wc(wc>0))
        else
            figure(ff);stem3(xc(wc>0),yc(wc>0),wc(wc>0))
        end
    end
    axis square;axis equal;axis([min(xc) max(xc) min(yc) max(yc) 0 1])
    view([-56.5 16])
    ff=ff+1;
    xlabel('x');ylabel('y');zlabel('w')
end
if elplt(3)==1 & sp==1
    figure(ff);surf(x0-max(x0)/2,y0-max(y0)/2,reshape(w,Ny,Nx))
    shading interp
    axis square;axis equal;axis([min(xc) max(xc) min(yc) max(yc) -1.1 1.1])
    xlabel('x');ylabel('y');zlabel('w')
    ff=ff+1;
end

%%%%%%%%%%%%  plot array factor  %%%%%%%%%%%%
if azel==1
    if afplt(1)==1
        figure(ff);surf(xx,yy,afdbp);shading faceted;axis('square'); colormap([1 1 1]);%([1 .6:-.01: 0]'*[1 1 1]);
        axis([-1 1 -1 1 botplt mafdb]);zlabel('Directivity (dB)')
        set(gca,'ytick',[]);set(gca,'xtick',[])
        hold on
        line([0 0],[-1 -1.2],[botplt botplt]);line([-1.2 -1],[0 0],[botplt botplt])
        line([-.04 0],[-1.1 -1.2],[botplt botplt]);line([.04 0],[-1.1 -1.2],[botplt botplt])
        line([-1.2 -1.1],[0 0.04 ],[botplt botplt]);line([-1.2 -1.1],[0 -0.04],[botplt botplt])
        line(-[1.1*cos([37:50]*pi/180)],-[1.1*sin([37:50]*pi/180)],.95*botplt*ones(1,14),'Color','r')
        line(-[1.09*cos(.82) 1.1*cos(.87) 1.135*cos(.82)],-[1.1*sin(.82)-.04 1.1*sin(.87) 1.1*sin(.82)+.02],.95*botplt*ones(1,3),'Color','r')
        text(-0.05,-1.35,botplt,'\theta');text(-1.4,0,botplt,'\theta')
        text(-1.12,-1.08,botplt+.1*bot,'\phi')
        hold off
        ff=ff+1;
    end
    if afplt(2)==1
        afdb2=afdbp+bot+Ddb;
        figure(ff);surf(samps,samps,afdb2);colormap([1 1 1]);axis('square');axis off;axis([-1 1 -1 1 min(min(afdb2)) max(max(afdb2))])
        ff=ff+1;
    end
    % plots in principal planes
    if afplt(3)==1
        % phi=0
        n4=4*nang;
        t6=0:90/(n4-1):90;;
        u0=sin(t6*pi/180);
        af0=w*exp(j*k*(x'*u0));
        af0a=abs(af0); maxaf0=max(af0a);
        af0db=20*log10(af0a/maxaf0)+Ddb;
        figure(ff);plot(t6,af0db);
        xlabel('\theta (\phi=0)');ylabel('AF (dB)');axis([0 90 -50+Ddb 0+Ddb]);ff=ff+1;
        set(gca,'XTick',0:45:90)
        set(gca,'XTickLabel',{'0','45','90'})
        % phi=90
        af0=w*exp(j*k*(y'*u0));
        af0a=abs(af0); maxaf0=max(af0a);
        af0db=20*log10(af0a/maxaf0)+Ddb;
        figure(ff);plot(t6,af0db);
        xlabel('\theta (\phi=90)');ylabel('AF (dB)');axis([0 90 -50+Ddb 0+Ddb]);ff=ff+1;
        set(gca,'XTick',0:45:90)
        set(gca,'XTickLabel',{'0','45','90'})
    end
else
    if afplt(4)==1
        figure(ff);surf(afdbp.*cos(phir).*sin(ther),afdbp.*sin(phir).*sin(ther),afdbp.*cos(ther))
        axis off; axis square; axis equal;colormap([1 1 1]);
        ff=ff+1;
    end
    if afplt(5)==1
        figure(ff);surf(the,phi,afdbp);axis('square');colormap([1 1 1]);
        xlabel('\theta');ylabel('\phi')
        ff=ff+1;
    end
end

if Nx*Ny==1
    figure(ff);surf(afr.*cos(phir).*sin(ther),afr.*sin(phir).*sin(ther),afr.*cos(ther))
    axis off; axis square; axis equal;colormap([1 1 1]);
end
%%%%%%%%%%%%  array factor information  %%%%%%%%%%%%
disp(['D=' num2str(D)])
disp(['D in dB=' num2str(Ddb)])
disp(['taper eff=' num2str(te)])
disp(['Nc=' num2str(Nc)])